/**
 ******************************************************************************
 * @file    easyspin.c
 * @author  EMEA AMS-IPD Marketing & Application, Prague - VE
 * @version V1.0.1
 * @date    June-2012
 * @brief   easySPIN (L6474) product related routines
 ******************************************************************************
 * @copy
 *
 * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
 * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
 * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
 * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
 * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
 * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
 *
 * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
 */

/* Includes ------------------------------------------------------------------*/
#include "easyspin.h"
#include "hal.h"


/**
 * @brief  Initializes easySPIN related peripherals, disables powerstage and releases reset pin
 * @param  None
 * @retval None
 */
void easySPIN_Init(void) 
{
    /* Disable easySPIN powerstage */
    easySPIN_Disable();
}

/**
 * @brief  Fills-in easySPIN configuration structure with default values.
 * @param  Structure address (pointer to struct)
 * @retval None
 */
void easySPIN_Regs_Struct_Reset( easySPIN_RegsStruct_TypeDef* easySPIN_RegsStruct) {
	easySPIN_RegsStruct->ABS_POS = 0x00;
	easySPIN_RegsStruct->EL_POS = 0x00;
	easySPIN_RegsStruct->MARK = 0x00;

	easySPIN_RegsStruct->TVAL = 0x00;
	easySPIN_RegsStruct->T_FAST = easySPIN_TOFF_FAST_8_0_us
			| easySPIN_FAST_STEP_8_0_us;
	easySPIN_RegsStruct->TON_MIN = 0x00;
	easySPIN_RegsStruct->TOFF_MIN = 0x01;

	/* OCD_TH register setup */
	easySPIN_RegsStruct->OCD_TH = easySPIN_OCD_TH_375mA;

	/* STEP_MODE register */
	easySPIN_RegsStruct->STEP_MODE = easySPIN_STEP_SEL_1
			| easySPIN_SYNC_SEL_1_2;

	/* ALARM_EN register setup */
	easySPIN_RegsStruct->ALARM_EN = easySPIN_ALARM_EN_OVERCURRENT
			| easySPIN_ALARM_EN_THERMAL_SHUTDOWN
			| easySPIN_ALARM_EN_THERMAL_WARNING
			| easySPIN_ALARM_EN_UNDERVOLTAGE | easySPIN_ALARM_EN_SW_TURN_ON
			| easySPIN_ALARM_EN_WRONG_NPERF_CMD;

	/* CONFIG register setup */
	easySPIN_RegsStruct->CONFIG = easySPIN_CONFIG_INT_16MHZ
			| easySPIN_CONFIG_EN_TQREG_INT_REG | easySPIN_CONFIG_OC_SD_ENABLE
			| easySPIN_CONFIG_SR_180V_us | easySPIN_CONFIG_TSW_8_us;
}

/**
 * @brief  Configures easySPIN internal registers with values in the config structure.
 * @param  Configuration structure address (pointer to configuration structure)
 * @retval None
 */
void easySPIN_Registers_Set(easySPIN_RegsStruct_TypeDef* easySPIN_RegsStruct) 
{
	easySPIN_SetParam(easySPIN_ABS_POS, easySPIN_RegsStruct->ABS_POS);
	easySPIN_SetParam(easySPIN_EL_POS, easySPIN_RegsStruct->EL_POS);
	easySPIN_SetParam(easySPIN_MARK, easySPIN_RegsStruct->MARK);
	easySPIN_SetParam(easySPIN_TVAL, easySPIN_RegsStruct->TVAL);
	easySPIN_SetParam(easySPIN_T_FAST, easySPIN_RegsStruct->T_FAST);
	easySPIN_SetParam(easySPIN_TON_MIN, easySPIN_RegsStruct->TON_MIN);
	easySPIN_SetParam(easySPIN_TOFF_MIN, easySPIN_RegsStruct->TOFF_MIN);
	easySPIN_SetParam(easySPIN_OCD_TH, easySPIN_RegsStruct->OCD_TH);
	easySPIN_SetParam(easySPIN_STEP_MODE, easySPIN_RegsStruct->STEP_MODE);
	easySPIN_SetParam(easySPIN_ALARM_EN, easySPIN_RegsStruct->ALARM_EN);
	easySPIN_SetParam(easySPIN_CONFIG, easySPIN_RegsStruct->CONFIG);
}

/* Application Commands implementation ----------------------------------------*/

/**  
 * @brief  Issues easySPIN NOP command.
 * @param  None
 * @retval None
 */
void easySPIN_Nop(void)
{
	/* Send NOP operation code to easySPIN */
	hal_spi_write_byte(easySPIN_NOP);
}

/**
 * @brief  Issues easySPIN SetParam command.
 * @param  easySPIN register address, value to be set
 * @retval None
 */
void easySPIN_SetParam(easySPIN_Registers_TypeDef param, uint32_t value) 
{
	/* Send SetParam operation code to easySPIN */
	hal_spi_write_byte(easySPIN_SET_PARAM | param);
	switch (param) {
	case easySPIN_ABS_POS:
		;
	case easySPIN_MARK:
		/* Send parameter - byte 2 to easySPIN */
		hal_spi_write_byte((uint8_t) (value >> 16));
	case easySPIN_EL_POS:
		;
	case easySPIN_CONFIG:
		;
	case easySPIN_STATUS:
		/* Send parameter - byte 1 to easySPIN */
		hal_spi_write_byte((uint8_t) (value >> 8));
	default:
		/* Send parameter - byte 0 to easySPIN */
		hal_spi_write_byte((uint8_t) (value));
	}
}

/**
 * @brief  Issues easySPIN GetParam command.
 * @param  easySPIN register address
 * @retval Register value - 1 to 3 bytes (depends on register)
 */
uint32_t easySPIN_GetParam(easySPIN_Registers_TypeDef param) 
{
	uint32_t temp = 0;
	uint32_t rx = 0;

	/* Send GetParam operation code to easySPIN */
	temp = hal_spi_write_byte(easySPIN_GET_PARAM | param);
	/* MSB which should be 0 */
	temp = temp << 24;
	rx |= temp;
	switch (param) {
	case easySPIN_ABS_POS:
		;
	case easySPIN_MARK:
		temp = hal_spi_write_byte((uint8_t) (0x00));
		temp = temp << 16;
		rx |= temp;
	case easySPIN_EL_POS:
		;
	case easySPIN_CONFIG:
		;
	case easySPIN_STATUS:
		temp = hal_spi_write_byte((uint8_t) (0x00));
		temp = temp << 8;
		rx |= temp;
	default:
		temp = hal_spi_write_byte((uint8_t) (0x00));
		rx |= temp;
	}
	return rx;
}

/**
 * @brief Issues easySPIN Enable command.
 * @param  None
 * @retval None
 */
void easySPIN_Enable(void) 
{
	/* Send Enable operation code to easySPIN */
	hal_spi_write_byte(easySPIN_ENABLE);
}

/**
 * @brief Issues easySPIN Disable command.
 * @param  None
 * @retval None
 */
void easySPIN_Disable(void) 
{
	/* Send Disable operation code to easySPIN */
	hal_spi_write_byte(easySPIN_DISABLE);
}

/**
 * @brief  Issues easySPIN GetStatus command.
 * @param  None
 * @retval Status Register content
 */
uint16_t easySPIN_Get_Status(void) 
{
	uint16_t temp = 0;
	uint16_t rx = 0;

	/* Send GetStatus operation code to easySPIN */
	hal_spi_write_byte(easySPIN_GET_STATUS);
	/* Send zero byte / receive MSByte from easySPIN */
	temp = hal_spi_write_byte((uint8_t) (0x00));
	temp <<= 8;
	rx |= temp;
	/* Send zero byte / receive LSByte from easySPIN */
	temp = hal_spi_write_byte((uint8_t) (0x00));
	rx |= temp;
	return rx;
}


/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
