#include "system.h"
#include "hal.h"
#include "easyspin.h"
#include "storage.h"


#define ENABLED_THR     100

typedef struct
{
    easySPIN_Registers_TypeDef reg;
    char                       name[10];
    int                        configure;
    uint32_t                   default_val;
}L6474_regs_desc_t;

const L6474_regs_desc_t regs_desc[] =
{
    { easySPIN_ABS_POS,     "ABS_POS" ,     0,  0},
    { easySPIN_EL_POS,      "EL_POS",       0,  0},
    { easySPIN_MARK,        "MARK",         0,  0},
    { easySPIN_TVAL,        "TVAL",         2,  0x29 /* 1.3125A */},
    { easySPIN_T_FAST,      "T_FAST",       1,  easySPIN_TOFF_FAST_5_0_us | easySPIN_FAST_STEP_1_0_us },
    { easySPIN_TON_MIN,     "TON_MIN",      1,  0x29 /* 20.5 us */},
    { easySPIN_TOFF_MIN,    "TOFF_MIN",     1,  0x29 /* 20.5 us */},
    { easySPIN_ADC_OUT,     "ADC_OUT",      0,  0 },
    { easySPIN_OCD_TH,      "OCD_TH",       1,  8 /* 3.38A */ },
    { easySPIN_STEP_MODE,   "STEP_MODE",    2,  easySPIN_STEP_SEL_1_16| easySPIN_SYNC_SEL_8 },
    { easySPIN_ALARM_EN,    "ALARM_EN",     1,  easySPIN_ALARM_EN_OVERCURRENT|easySPIN_ALARM_EN_THERMAL_SHUTDOWN| easySPIN_ALARM_EN_THERMAL_WARNING| easySPIN_ALARM_EN_UNDERVOLTAGE},
    { easySPIN_CONFIG,      "CONFIG",       1,  easySPIN_CONFIG_INT_16MHZ| easySPIN_CONFIG_EN_TQREG_INT_REG| easySPIN_CONFIG_OC_SD_ENABLE| easySPIN_CONFIG_SR_290V_us| easySPIN_CONFIG_TSW_8_us},
    { easySPIN_STATUS,      "STATUS",       0,  0 },
    { 0xFF,"",0,0}
};


volatile int reconfigure = 0;

void easySPIN_configure(uint16_t inputs)
{
    uint16_t     tmp;
    int          ii;
    
    /* Initialize L6474, also disables power stage*/
    easySPIN_Init();

    if( (l6474_storage_data.registers[easySPIN_TVAL] & REG_FORCE) == 0)
    {
        /* TVAL register setup   - from DSW */
        l6474_storage_data.registers[easySPIN_TVAL] = ((inputs & 0x0F) + 1 ) * 8;
    }

    if( (l6474_storage_data.registers[easySPIN_STEP_MODE] & REG_FORCE) == 0)
    {

        /* STEP_MODE register setup  - from DSW*/
        tmp = (inputs >> 4) & 0x02;
        switch(tmp)
        {
            case 0:  l6474_storage_data.registers[easySPIN_STEP_MODE] =  easySPIN_STEP_SEL_1_2| easySPIN_SYNC_SEL_8;break;
            case 1:  l6474_storage_data.registers[easySPIN_STEP_MODE] =  easySPIN_STEP_SEL_1_4| easySPIN_SYNC_SEL_8;break;
            case 2:  l6474_storage_data.registers[easySPIN_STEP_MODE] =  easySPIN_STEP_SEL_1_8| easySPIN_SYNC_SEL_8;break;
            case 3:  l6474_storage_data.registers[easySPIN_STEP_MODE] = easySPIN_STEP_SEL_1_16| easySPIN_SYNC_SEL_8;break;
        }

    }

    ii = 0;

    while(regs_desc[ii].reg != 0xFF)
    {
        if(regs_desc[ii].configure != 0)
        {
            if((regs_desc[ii].configure == 2) || ((l6474_storage_data.registers[regs_desc[ii].reg] & REG_FORCE)!= 0))
            {
                easySPIN_SetParam(regs_desc[ii].reg, l6474_storage_data.registers[regs_desc[ii].reg] & ~ REG_FORCE);
            }
            else
            {
                easySPIN_SetParam(regs_desc[ii].reg,regs_desc[ii].default_val);
            }
        }
        ii++;
    }


}

void easySPIN_dump(void)
{
    int      ii;
    uint32_t value;
    char     buffer[64];

    hal_uart_puts("\r\n\r\nL6474 controller board. Copyright cnc@kartech.biz\r\n");

    ii = 0;
    while(regs_desc[ii].reg != 0xFF)
    {
        value = easySPIN_GetParam(regs_desc[ii].reg);
        sprintf(buffer,"Reg 0x%02X (%10s)= 0x%08lX\r\n",(unsigned int)regs_desc[ii].reg,regs_desc[ii].name,value);
        hal_uart_puts(buffer);
        ii++;
    }

}


void easySPIN_dump_status(void)
{
    uint32_t status;
    char     buffer[64];

    status = easySPIN_Get_Status();
    sprintf(buffer,"Status = 0x%08lX\r\n",status);
    hal_uart_puts(buffer);
}

int main(void)
{
    uint16_t     inputs;
    uint16_t     old_inputs;
    uint16_t     stat_cntr = 0;


    uint32_t     status;
    int          enabled_cnt = 0;

    hal_oscilator_init();
    hal_ioport_init();
    hal_uart_init();

    hal_uart_puts("\r\n\r\nL6474 controller board. Copyright cnc@kartech.biz\r\n");

    hal_spi_init();
    storage_init();


    /*  Read DSW */
    old_inputs = inputs = hal_read_inputs();

#if 1
    // Check if SPI works reading default values
    easySPIN_dump();
#endif
   
    easySPIN_configure(inputs);
    status = easySPIN_Get_Status();

    easySPIN_dump();

    while(1)
    {
        inputs = hal_read_inputs();

        if( (inputs & 0x8000)== 0)
        {
            if(enabled_cnt < ENABLED_THR)
            {
               enabled_cnt++;
               if(enabled_cnt == ENABLED_THR)
               {
                  /* Enable power stage */
                  easySPIN_Enable();
                  easySPIN_dump();
               }
            }
        }
        else
        {
            if(enabled_cnt > 0)
            {
               enabled_cnt--;
               if(enabled_cnt == 0)
               {
                    easySPIN_Disable();
                    easySPIN_dump();
               }
            }
        }

        if( (reconfigure!= 0) || ( inputs != old_inputs))
        {
            reconfigure = 0;
            old_inputs  = inputs;
            enabled_cnt = 0;

            easySPIN_configure(inputs);
            easySPIN_dump();
        }

        if( (stat_cntr % 10000) == 9999)
        {
            easySPIN_dump_status();
        }
        stat_cntr++;
    }

    return 0; /* Note - never comes here */

}


void process_request(char * buffer)
{
    int reg;
    int value;
    int cnt;

    cnt = sscanf(buffer,"%*c%X=%X",&reg,&value);

    if(cnt == 1)
    {
        if(reg < REG_CNT)
        {
            // Clear override
            l6474_storage_data.registers[reg] = 0;
            reconfigure = 1;
        }
    }
    else if(cnt ==2)
    {
        if(reg < REG_CNT)
        {
            // Set override
            l6474_storage_data.registers[reg] = value | REG_FORCE ;
            storage_update();
            reconfigure = 1;
        }
    }


}